var DocumentScheme = require("./DocumentScheme.js");

var billon = {

    // Attributes
    SoapService: require("./SoapService.js"),

    // Functions
    uniqueId: require('node-uuid').v1
};

billon.Shop = function() {
    this.Merchant = function(requestData, items, delivery_option, cb) {
        cb({});
    };
    this.App24 = false;

    this.User = function(cb) {
        cb(false);
    };

    this.Products = function(cb) {
        cb(false);
    };

    this.Product = function(cb) {
        cb(false);
    };

    this.ProductBillon = function(requestData, item, cb) {
        cb(item);
    };

    this.ProductDelivery = function(requestData, items, cb) {
        cb(false);
    };

    this.ProductDownload = function(requestData, items, delivery, document, cb) {
        cb(false);
    };

    this.PaymentTitle = function(cb) {
        cb(false);
    };

    this.Currency = function(requestData, cb) {
        cb('PLN');
    };
    
    this.PlatePayment = function(requestData, cb) {
        cb('', 'NORMAL');
    };

    this.DocumentParse = function(requestData, document, cb) {
        cb(document);
    };
    
    this.Plates = function(requestData, cb) {
        cb(false);
    };
    
    this.MaxTotalAmount = function(requestData, cb) {
        cb(0);
    };
    
    this.Duration = function(requestData, cb) {
        cb(0);
    };
    
    this.MaxPaymentsInSeries = function(requestData, cb) {
        cb(0);
    };
    
    this.MerchantBrandName = function(requestData, cb) {
        cb(false);
    };

    this.OnFinishedPayment = function() {
        console.log("Finished some payment.");
    };
    
    this.OnReceivedPlatePaymentSeriesStatus = function() {};
    
    this.OnAcceptedPlatePaymentSeries = function() {};

};

// HELPERS
var isFunction = function(func) {
    if (typeof func !== "function") {
        return false;
    }

    return true;
};

var deliveryOptions = {
    "NOT_NEEDED": 0,
    "OPTIONAL": 1,
    "OBLIGATORY": 2
};

var uids = [];

var taskTracker = {};

var getMerchantBrandName = function(headers, mbn) {
   if (mbn) {
      return mbn;
   } else {
      var url = require('url');
      return url.parse(headers['origin'] ? headers['origin'] : headers['referer']).hostname;
   }
};

var addPeerLocationIfAvailable = function(user, params) {
    if (typeof user.confirmationCode1 != 'undefined' && user.confirmationCode1 != null && typeof user.confirmationCode1 != 'undefined' && user.confirmationCode1 != null) {
      if (!user.userLocation || !user.userLocation.machineIdHex || !user.userLocation.ip || !user.userLocation.tcp || !user.userLocation.udp) {
         user.userLocation = null;
      }
      if (!user.buddyLocation || !user.buddyLocation.machineIdHex || !user.buddyLocation.ip || !user.buddyLocation.tcp || !user.buddyLocation.udp) {
         user.buddyLocation = null;
      }
      params.peerLocation = user;
    }
};

// ERRORS
var errs = {
    ERR_INVALID_BODY: {
        err: "ERR_INVALID_BODY",
        description: "Missing body."
    },
    ERR_INVALID_QUERY: {
        err: "ERR_INVALID_QUERY",
        description: "Missing query."
    },
    ERR_INVALID_QUERY_DATA: {
        err: "ERR_INVALID_QUERY_DATA",
        description: "Data must be an object."
    },
    ERR_INVALID_PRODUCTS: {
        err: "ERR_INVALID_PRODUCTS",
        description: "Data.products must be an object."
    },
    ERR_INVALID_USER: {
        err: "ERR_INVALID_USER",
        description: "Data.user must be an object."
    },
    ERR_INVALID_WUI: {
        err: "ERR_INVALID_WUI",
        description: "Data.wui must be an object."
    },
    ERR_INVALID_USER_USERNAME: {
        err: "ERR_INVALID_USER_USERNAME",
        description: "Data.user.userName must be a string with minimum 4 characters length."
    },
    ERR_INVALID_TASKID: {
        err: "ERR_INVALID_TASKID",
        description: "Invalid taskId sent."
    },
    ERR_INVALID_UID: {
        err: "ERR_INVALID_UID",
        description: "Invalid uid sent."
    },
    ERR_TASK_DOES_NOT_EXIST: {
        err: "ERR_TASK_DOES_NOT_EXIST",
        description: "Task does not exist."
    },
    ERR_CANNOT_DOWNLOAD: {
        err: "ERR_CANNOT_DOWNLOAD",
        description: "Cannot download file."
    },
    ERR_INVALID_AMOUNT: {
        err: "ERR_INVALID_AMOUNT",
        description: "Invalid amount."
    }
};

var plateColors = {'PLATE_ALPHA': '44B905', 'PLATE_BETA': '00ADFF', 'PLATE_GAMMA': 'B466CB', 'PLATE_DELTA': 'F6891C', 'PLATE_EPSILON': 'EC2B41'};

billon.Shop.prototype = {
    merchant: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.merchant parameter must be a function.");
            return false;
        }

        this.Merchant = func;
        return this;
    },
    app24: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.app24 parameter must be a function.");
            return false;
        }

        this.App24 = func;
        return this;
    },
    user: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.user parameter must be a function.");
            return false;
        }

        this.User = func;
        return this;
    },
    products: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.products parameter must be a function.");
            return false;
        }

        this.Products = func;
        return this;
    },
    product: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.product parameter must be a function.");
            return false;
        }

        this.Product = func;
        return this;
    },
    productBillon: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.productBillon parameter must be a function.");
            return false;
        }

        this.ProductBillon = func;
        return this;
    },
    productDelivery: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.productDelivery parameter must be a function.");
            return false;
        }

        this.ProductDelivery = func;
        return this;
    },
    productDownload: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.productDownload parameter must be a function.");
            return false;
        }

        this.ProductDownload = func;
        return this;
    },
    currency: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.currency parameter must be a function.");
            return this;
        }

        this.Currency = func;
        return this;
    },
    platePayment: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.platePayment parameter must be a function.");
            return this;
        }

        this.PlatePayment = func;
        return this;
    },
    documentParse: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.documentParse parameter must be a function.");
            return false;
        }

        this.DocumentParse = func;
        return this;
    },
    paymentTitle: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.paymentTitle parameter must be a function.");
            return false;
        }

        this.PaymentTitle = func;
        return this;
    },
    plates: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.plates parameter must be a function.");
            return this;
        }

        this.Plates = func;
        return this;
    },
    maxTotalAmount: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.maxTotalAmount parameter must be a function.");
            return this;
        }

        this.MaxTotalAmount = func;
        return this;
    },
    duration: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.duration parameter must be a function.");
            return this;
        }

        this.Duration = func;
        return this;
    },
    maxPaymentsInSeries: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.maxPaymentsInSeries parameter must be a function.");
            return this;
        }

        this.MaxPaymentsInSeries = func;
        return this;
    },
    merchantBrandName: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.merchantBrandName parameter must be a function.");
            return this;
        }

        this.MerchantBrandName = func;
        return this;
    },
    onFinishedPayment: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.onFinishedPayment parameter must be a function.");
            return false;
        }

        this.OnFinishedPayment = func;
        return this;
    },
    onReceivedPlatePaymentSeriesStatus: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.onReceivedPlatePaymentSeriesStatus parameter must be a function.");
            return false;
        }

        this.OnReceivedPlatePaymentSeriesStatus = func;
        return this;
    },
    onAcceptedPlatePaymentSeries: function(func) {
        if (!isFunction(func)) {
            console.log("Shop.onAcceptedPlatePaymentSeries parameter must be a function.");
            return false;
        }

        this.OnAcceptedPlatePaymentSeries = func;
        return this;
    },
    donateInitiation: function(req, res) {
        var self = this;

        if (typeof req.body !== "object") {
            return res.send(errs.ERR_INVALID_QUERY);
        }

        var requestData = req.body;

        if (typeof requestData.user !== "object") {
            return res.send(errs.ERR_INVALID_USER);
        }

        if (typeof requestData.amount !== "number") {
            return res.send(errs.ERR_INVALID_AMOUNT);
        }

        if (typeof requestData.user.userName !== "string" || requestData.user.userName.length < 4) {
            return res.send(errs.ERR_INVALID_USER_USERNAME);
        }

        var paymentData = {
            user: requestData.user,
            amount: requestData.amount
        };

        var requestInvoiceData = "NOT_NEEDED";
        var requestDeliveryData = "NOT_NEEDED";
        self.App24(requestData, null, null, function(wui) {
            var document = DocumentScheme();
            self.DocumentParse(requestData, document, function(parsedDocument) {
                if (parsedDocument !== false) {
                    document = parsedDocument;
                }

                // Wczytanie danych sprzedawcy.
                self.Merchant(requestData, document.sales_document_data.items, document.sales_document_data.delivery_option, function(merchant) {
                    var uid = billon.uniqueId();
                    document.sales_document_data.seller_data = merchant;
                    document.sales_document_data.sales_document_id = uid;

                    // Ustalenie daty.
                    document.sales_document_data.sales_document_date = new Date().getTime();
                    document.sales_document_data.trade_date = new Date().getTime();
                    document.sales_document_data.total_amount_gross = paymentData.amount;

                    self.PaymentTitle(requestData, document.sales_document_data.items, document.sales_document_data.delivery_option, function(title) {
                        self.Currency(requestData, function(currency) {
                           self.MerchantBrandName(requestData, function(mbn) {
                            var data = {
                                WUI: wui,
                                params: {
                                    type: "NORMAL",
                                    amount: {
                                        amount: paymentData.amount,
                                        currency: currency
                                    },
                                    requestData: {
                                        username: paymentData.user.userName,
                                        uniqueTransferId: document.sales_document_data.sales_document_id,
                                        title: title,
                                        merchantBrandName: getMerchantBrandName(req.headers, mbn),
                                        requestInvoiceData: requestInvoiceData,
                                        requestDeliveryData: requestDeliveryData,
                                        description: document ? JSON.stringify(document) : null
                                    }
                                }
                            };
                            addPeerLocationIfAvailable(paymentData.user, data.params);
                            billon.SoapService.call.requestPaymentInitiation(data, function(requestResponse) {
                                var copy = _.clone(requestResponse);
                                if (requestResponse.requestStatus === "SUCCESS") {
                                    taskTracker[uid] = {
                                        taskId: requestResponse.taskId,
                                        WUI: wui,
                                        paymentData: paymentData
                                    };
                                    var interval = setInterval(function() {
                                        billon.SoapService.call.getTaskStatus({
                                            WUI: wui,
                                            params: {
                                                taskId: requestResponse.taskId
                                            }
                                        }, function(taskResponse) {
                                            if (taskResponse.requestStatus !== "SUCCESS") {
                                                setTimeout(function() {
                                                    delete taskTracker[uid];
                                                }, 30000);
                                                return clearInterval(interval);
                                            }

                                            if (taskResponse.status.split("_")[0] === "FINISHED" || taskResponse.progressPercent === 100 || taskResponse.progressPercent === "100") {
                                                clearInterval(interval);
                                                setTimeout(function() {
                                                    delete taskTracker[uid];
                                                }, 30000);
                                            }

                                            if (taskResponse.status === "FINISHED_OK") {
                                                self.OnFinishedPayment(paymentData);
                                            }

                                        });
                                    }, 5000);
                                    copy.taskId = uid;
                                }

                                res.json(copy);
                            });
                           });
                        });
                    });
                });
            });
        });
    },
    paymentInitiation: function(req, res) {
        var self = this;
        if (typeof req.body !== "object") {
            return res.send(errs.ERR_INVALID_BODY);
        }

        var requestData = req.body,
            paymentData = {};

        self.User(requestData, function(user) {
            if (typeof user !== "object") {
                return res.send(errs.ERR_INVALID_USER);
            }

            if (typeof user.userName !== "string" || user.userName.length < 4) {
                return res.send(errs.ERR_INVALID_USER_USERNAME);
            }

            paymentData.user = user;

            self.Products(requestData, function(products) {
                if (typeof products !== "object") {
                    return res.send(errs.ERR_INVALID_PRODUCTS);
                }

                paymentData.products = products;

                var document = DocumentScheme();

                self.DocumentParse(requestData, document, function(parsedDocument) {
                    if (parsedDocument !== false) {
                        document = parsedDocument;
                    }

                    var requestInvoiceData = "NOT_NEEDED";
                    var requestDeliveryData = "NOT_NEEDED";

                    // Wczytanie produktów.
                    var itemsLoaded = 0;
                    paymentData.items = [];
                    var productsLength = paymentData.products.length;
                    var onItemReceived = function(item) {
                        paymentData.items.push(item);
                        self.ProductBillon(requestData, item, function(currentItem) {
                            if (typeof currentItem !== "object") {
                                console.log("Empty item.");
                                return;
                            }
                            currentItem.unit_price_net = parseInt(currentItem.unit_price_net * 1000);
                            currentItem.amount_net = currentItem.unit_price_net * currentItem.quantity;
                            document.sales_document_data.total_amount_net += currentItem.amount_net;
                            currentItem.amount_gross = currentItem.amount_gross * 1000;
                            document.sales_document_data.total_amount_gross += currentItem.amount_gross * currentItem.quantity;
                            // Sprawdzenie czy potrzeba danych do faktury lub dostawy.
                            if (deliveryOptions[currentItem.requestInvoiceData] > deliveryOptions[requestInvoiceData]) {
                                requestInvoiceData = currentItem.requestInvoiceData;
                            }

                            if (deliveryOptions[currentItem.requestDeliveryData] > deliveryOptions[requestDeliveryData]) {
                                requestDeliveryData = currentItem.requestDeliveryData;
                            }

                            // Pomnożenie vat'u do danych dokumentu.
                            currentItem.vat.amount *= 1000;

                            // Dodanie produktu do listy.
                            document.sales_document_data.items.push(currentItem);

                            itemsLoaded++;
                            if (itemsLoaded === productsLength) {
                                onItemsLoaded();
                            }
                        });
                    };

                    for (var i = 0; i < productsLength; i++) {
                        var itemId = paymentData.products[i];
                        self.Product(requestData, itemId, onItemReceived);
                    }

                    function onItemsLoaded() {

                        // Wczytanie opcji dostawy.
                        self.ProductDelivery(requestData, document.sales_document_data.items, function(delivery) {
                            document.sales_document_data.delivery_option = delivery;

                            // Wczytanie danych sprzedawcy.
                            self.Merchant(requestData, document.sales_document_data.items, document.sales_document_data.delivery_option, function(merchant) {
                                var uid = billon.uniqueId();
                                document.sales_document_data.seller_data = merchant;
                                document.sales_document_data.sales_document_id = uid;

                                // Ustalenie daty.
                                document.sales_document_data.sales_document_date = new Date().getTime();
                                document.sales_document_data.trade_date = new Date().getTime();

                                self.App24(requestData, document.sales_document_data.items, document.sales_document_data.delivery_option, function(wui) {
                                    if (typeof wui !== "object") {
                                        return res.send(errs.ERR_INVALID_WUI);
                                    }
                                    self.PaymentTitle(requestData, document.sales_document_data.items, document.sales_document_data.delivery_option, function(title) {
                                        self.Currency(requestData, function(c) {
                                          self.PlatePayment(requestData, function(spid, t) {
                                             self.MerchantBrandName(requestData, function(mbn) {
                                               var keyCodeArr = [];
                                               var counter = 0;
                                               var keyCodeItems = _.filter(paymentData.items, function(item) {
                                                   return item.KeyCodes;
                                               });
                                               var keyCodeItemsLength = keyCodeItems.length;

                                               var errorCalled = false;
                                               var onKeyCodeDownload = function(download) {
                                                   if (errorCalled) {
                                                       return;
                                                   }

                                                   if (download.err) {
                                                       errorCalled = true;
                                                       return res.json(errs.ERR_CANNOT_DOWNLOAD);
                                                   }

                                                   if (!download || download.length === 0) {
                                                       return res.json(errs.ERR_CANNOT_DOWNLOAD);
                                                   }

                                                   // TODO: Obsłużyć wiele kodów jako 1 produkt.
                                                   KeyCode.update({
                                                       id: download[0].id
                                                   }, {
                                                       Status: "blocked"
                                                   }).exec(function(err, updated) {
                                                       if (err) {
                                                           console.log(err);
                                                           return;
                                                       }

                                                       if (typeof updated !== "undefined") {
                                                           keyCodeArr.push(updated[0]);
                                                       }

                                                       counter++;
                                                       if (keyCodeItemsLength === counter) {
                                                           onDownloadedFinished();
                                                       }
                                                   });
                                               };

                                               if (keyCodeItemsLength === 0) {
                                                   return self.ProductDownload(requestData, paymentData.items, delivery, document, onDownloadedFinished);
                                               }

                                               for (var i = 0; i < keyCodeItemsLength; i++) {
                                                   self.ProductDownload(requestData, paymentData.items, delivery, document, onKeyCodeDownload);
                                               }

                                               function onDownloadedFinished(downloadArr) {
                                                   if (t) {
                                                      t = t.toUpperCase();
                                                      if (t == 'ALPHA' || t == 'BETA' || t == 'GAMMA' || t == 'DELTA' || t == 'EPSILON') {
                                                         t = 'PLATE_' + t;
                                                      }
                                                   }
                                                   var data = {
                                                       WUI: wui,
                                                       params: {
                                                           type: t ? t : "NORMAL",
                                                           amount: {
                                                               amount: document.sales_document_data.total_amount_gross,
                                                               currency: c
                                                           },
                                                           requestData: {
                                                               username: paymentData.user.userName,
                                                               uniqueTransferId: uid,
                                                               title: title,
                                                               merchantBrandName: getMerchantBrandName(req.headers, mbn),
                                                               requestInvoiceData: requestInvoiceData,
                                                               requestDeliveryData: requestDeliveryData,
                                                               description: JSON.stringify(document)
                                                           }
                                                       }
                                                   };
                                                   if (t && t.indexOf('PLATE_') == 0) {
                                                      data.params.requestData.plateColorHex = plateColors[t];
                                                   }
                                                   if (spid) {
                                                      data.params.seriesId = spid;
                                                   }
                                                   addPeerLocationIfAvailable(paymentData.user, data.params);
                                                   billon.SoapService.call.requestPaymentInitiation(data, function(requestResponse) {
                                                       var onUpdated = function(err, updated) {
                                                           if (err) {
                                                               console.log(err);
                                                           }
                                                       };

                                                       var unBlockKeyCodes = function() {
                                                           for (var i = 0; i < keyCodeArr.length; i++) {
                                                               var keyCode = keyCodeArr[i];
                                                               KeyCode.update({
                                                                   id: keyCode.id
                                                               }, {
                                                                   Status: "available"
                                                               }).exec(onUpdated);
                                                           }
                                                       };

                                                       if (requestResponse.err || requestResponse.requestStatus !== "SUCCESS") {
                                                           unBlockKeyCodes();
                                                       }

                                                       var copy = _.clone(requestResponse);
                                                       if (requestResponse.requestStatus === "SUCCESS") {
                                                           taskTracker[uid] = {
                                                               requestData: requestData,
                                                               taskId: requestResponse.taskId,
                                                               WUI: wui,
                                                               items: paymentData.items,
                                                               delivery: document.sales_document_data.delivery_option,
                                                               document: document,
                                                               download: downloadArr,
                                                               keyCodes: keyCodeArr
                                                           };

                                                           var interval = setInterval(function() {
                                                               billon.SoapService.call.getTaskStatus({
                                                                   WUI: wui,
                                                                   params: {
                                                                       taskId: requestResponse.taskId
                                                                   }
                                                               }, function(taskResponse) {
                                                                   if (taskResponse.requestStatus !== "SUCCESS") {
                                                                       clearInterval(interval);
                                                                       setTimeout(function() {
                                                                           delete taskTracker[uid];
                                                                       }, 30000);

                                                                       unBlockKeyCodes();
                                                                       return;
                                                                   }

                                                                   var onUpdatedKey = function(err, updated) {
                                                                       if (err) {
                                                                           console.log(err);
                                                                       }
                                                                   };

                                                                   var keyCodeUsed;

                                                                   if (taskResponse.status.split("_")[0] === "FINISHED" || taskResponse.progressPercent === 100 
                                                                       || taskResponse.progressPercent === "100") {
                                                                       clearInterval(interval);
                                                                       setTimeout(function() {
                                                                           delete taskTracker[uid];
                                                                       }, 30000);

                                                                       if (taskResponse.status !== "FINISHED_OK") {
                                                                           for (var j = 0; j < keyCodeArr.length; j++) {
                                                                               keyCodeUsed = keyCodeArr[j];
                                                                               KeyCode.update({
                                                                                   id: keyCodeUsed.id
                                                                               }, {
                                                                                   Status: "available"
                                                                               }).exec(onUpdatedKey);
                                                                           }
                                                                       }

                                                                   }

                                                                   if (taskResponse.status === "FINISHED_OK") {
                                                                       for (var k = 0; k < keyCodeArr.length; k++) {
                                                                           keyCodeUsed = keyCodeArr[k];
                                                                           console.log("keyCodeUsed");
                                                                           console.log(keyCodeUsed);
                                                                           KeyCode.update({
                                                                               id: keyCodeUsed.id
                                                                           }, {
                                                                               Status: "sold"
                                                                           }).exec(onUpdatedKey);
                                                                       }

                                                                       document.additionalInfo = taskResponse.additionalInfo;
                                                                       self.OnFinishedPayment(document.sales_document_data.items, document.sales_document_data.delivery_option, document, requestData);
                                                                   }

                                                               });
                                                           }, 5000);
                                                           copy.taskId = uid;
                                                       }

                                                       res.json(copy);

                                                   });
                                               }
                                             });
                                          });
                                        });
                                    });
                                });
                            });
                        });
                    }
                });
            });
        });
    },
    platesList: function(req, res) {
        var self = this;
        if (self.platesList.arguments.length == 1) {
            res = req;
        }
        self.Plates({}, function(alpha, beta, gamma, delta, epsilon) {
            self.Currency({}, function(c) {
               res.json({amounts: [alpha, beta, gamma, delta, epsilon], currency: c});
            });
        });
    },
    platePaymentSeriesStart: function(req, res) {
        var self = this;
        if (typeof req.body !== "object") {
            return res.send(errs.ERR_INVALID_BODY);
        }
        var requestData = req.body,
            paymentData = {};
        self.User(requestData, function(user) {
            if (typeof user !== "object") {
                return res.send(errs.ERR_INVALID_USER);
            }

            if (typeof user.userName !== "string" || user.userName.length < 4) {
                return res.send(errs.ERR_INVALID_USER_USERNAME);
            }
            paymentData.user = user;
            self.Plates(requestData, function(alpha, beta, gamma, delta, epsilon) {
               var cpc = _.clone(plateColors);
               if (!beta) {
                  beta = alpha;
                  cpc.PLATE_BETA = cpc.PLATE_ALPHA;
                  alpha = 0;
               }
               if (!gamma) {
                  gamma = beta;
                  beta = alpha;
                  cpc.PLATE_GAMMA = cpc.PLATE_BETA;
                  cpc.PLATE_BETA = cpc.PLATE_ALPHA;
                  alpha = 0;
               }
               if (!delta) {
                  delta = gamma;
                  gamma = beta;
                  beta = alpha;
                  cpc.PLATE_DELTA = cpc.PLATE_GAMMA;
                  cpc.PLATE_GAMMA = cpc.PLATE_BETA;
                  cpc.PLATE_BETA = cpc.PLATE_ALPHA;
                  alpha = 0;
               }
               if (!epsilon) {
                  epsilon = delta;
                  delta = gamma;
                  gamma = beta;
                  beta = alpha;
                  cpc.PLATE_EPSILON = cpc.PLATE_DELTA;
                  cpc.PLATE_DELTA = cpc.PLATE_GAMMA;
                  cpc.PLATE_GAMMA = cpc.PLATE_BETA;
                  cpc.PLATE_BETA = cpc.PLATE_ALPHA;
                  alpha = 0;
               }
               var plates = {'alpha-price': alpha, 'beta-price': beta, 'gamma-price': gamma, 'delta-price': delta, 'epsilon-price': epsilon,
                             'alpha-color': cpc.PLATE_ALPHA, 'beta-color': cpc.PLATE_BETA, 'gamma-color': cpc.PLATE_GAMMA, 
                             'delta-color': cpc.PLATE_DELTA, 'epsilon-color': cpc.PLATE_EPSILON};
               self.App24(requestData, plates, null, function(wui) {
                        if (typeof wui !== "object") {
                           return res.send(errs.ERR_INVALID_WUI);
                        }
                        self.Currency(requestData, function(c) {
                           self.MaxTotalAmount(requestData, function(mta) {
                              self.Duration(requestData, function(d) {
                                 self.MaxPaymentsInSeries(requestData, function(mp) {
                                    self.MerchantBrandName(requestData, function(t) {
                                       var title = getMerchantBrandName(req.headers, t);
                                       var uid = billon.uniqueId();
                                       data = {WUI: wui, 
                                               params: {seriesData: {
                                                            description: ' ', 
                                                            username: paymentData.user.userName, 
                                                            maxTotalAmount: {amount: mta, currency: c, colour: 0}, 
                                                            maxPaymentsInSeries: mp,
                                                            seriesSecondDuration: d, 
                                                            plateParameters: plates}, 
                                                        type: 'PLATE_PAYMENT_SERIES', 
                                                        requestData: {
                                                            username: paymentData.user.userName,
                                                            uniqueTransferId: uid,
                                                            title: title,
                                                            merchantBrandName: title,
                                                            requestInvoiceData: 'NOT_NEEDED',
                                                            requestDeliveryData: 'NOT_NEEDED',
                                                            description: ' ', 
                                                            channelInfo: ' ', 
                                                            antiSpamCode: ' '}}};
                                       addPeerLocationIfAvailable(paymentData.user, data.params);
                                       billon.SoapService.call.startSeriesPayment(data, function(requestResponse) {
                                            var copy = _.clone(requestResponse);
                                            if (requestResponse.requestStatus === "SUCCESS") {
                                                taskTracker[uid] = {
                                                    taskId: requestResponse.taskId,
                                                    WUI: wui,
                                                    paymentData: paymentData
                                                };
                                                var interval = setInterval(function() {
                                                    billon.SoapService.call.getTaskStatus({
                                                        WUI: wui,
                                                        params: {
                                                            taskId: requestResponse.taskId
                                                        }
                                                    }, function(taskResponse) {
                                                        self.OnReceivedPlatePaymentSeriesStatus(taskResponse);
                                                        if (taskResponse.requestStatus !== "SUCCESS") {
                                                            setTimeout(function() {
                                                                delete taskTracker[uid];
                                                            }, 30000);
                                                            return clearInterval(interval);
                                                        }
                                                        var PaymentAccepted = taskResponse.status === "TRANSFERRING" || taskResponse.status === "TRANSFERING"
                                                                                                      || taskResponse.status === "SERIES_ACCEPTED";
                                                        if (PaymentAccepted || taskResponse.status.split("_")[0] === "FINISHED" || 
                                                            taskResponse.progressPercent === 100 || taskResponse.progressPercent === "100") {
                                                            clearInterval(interval);
                                                            setTimeout(function() {
                                                                delete taskTracker[uid];
                                                            }, 30000);
                                                        }
                                                        if (PaymentAccepted) {
                                                            self.OnAcceptedPlatePaymentSeries(paymentData);
                                                        }

                                                    });
                                                }, 5000);
                                                copy.taskId = uid;
                                            }
                                            res.json(copy);
                                       });
                                 });
                              });
                           });
                        });
               });
            });
         });
      });
    },
    paymentStatus: function(uid, cb) {
        if (typeof uid !== "string") {
            return cb(errs.ERR_INVALID_UID);
        }

        var self = this;
        var taskData = taskTracker[uid];
        if (typeof taskData === "undefined") {
            return cb(errs.ERR_TASK_DOES_NOT_EXIST);
        }

        billon.SoapService.call.getTaskStatus({
            WUI: taskData.WUI,
            params: {
                taskId: taskData.taskId
            }
        }, function(taskResponse) {

            if (typeof taskResponse.status === "string" && taskResponse.status === "FINISHED_OK") {
                var publicCodes = [];
                if (taskData && taskData.keyCodes) {
                    for (var i = 0; i < taskData.keyCodes.length; i++) {
                        publicCodes.push(taskData.keyCodes[i].Value);
                    }
                }
                taskResponse.agent = {
                    download: taskData.download,
                    keyCodes: publicCodes
                };

                if (cb) {
                    cb(taskResponse);
                }
            } else if (cb) {
                cb(taskResponse);
            }
        });
    }
};

module.exports = billon;
